#include <windows.h>
#include <iostream>
#include <sstream>
#include <string>
using namespace std;

#include "sys.h"
#include "debug.h"



//create named pipe
pipe_t* create_pipe()
{
    static int pipe_n=0;
    pipe_t *hp = new pipe_t;
    ostringstream s;
    s << "\\\\.\\pipe\\tgvcnv" << pipe_n++;
    hp->pipename = s.str();
    DEBUG_OUT << "Creating named pipe " << hp->pipename << endl;

    //create named pipe
    hp->hPipe=CreateNamedPipe(
        hp->pipename.c_str(),     // pipe name
        PIPE_ACCESS_DUPLEX,       // rw access
        PIPE_TYPE_BYTE |          // byte type pipe
        PIPE_READMODE_BYTE |      // byte-read mode
        PIPE_NOWAIT,              // non blocking mode (for async connect)
        1,                        // max. instances
        PIPESIZE,                 // output buffer size
        PIPESIZE,                 // input buffer size
        0,                        // client time-out
        NULL);                    // default security attribute

    //error?
    if(hp->hPipe==INVALID_HANDLE_VALUE)
    {
        cout << "Error: CreateNamedPipe failed." << endl;
        delete hp;
        return NULL;
    }
    DEBUG_OUT << "Pipe " << hp->pipename << " created." << endl;
    return hp;
}


//connect to named pipe
int connect_pipe(pipe_t *hp)
{
    //wait for connection from client, 5 sec timeout
    DEBUG_OUT << "Connecting to named pipe " << hp->pipename << endl;
    unsigned long t=GetTickCount();
    int conn=0;
    while(t+5000ul>GetTickCount())
    {
        conn=ConnectNamedPipe(hp->hPipe, NULL) ? 1 : (GetLastError()==ERROR_PIPE_CONNECTED);
        if(conn) break;
        Sleep(10);
    }

    //connected?
    if (!conn)
    {
        cout << "Error: no connection from ffmpeg." << endl;
        CloseHandle(hp->hPipe);
        return 0;
    }
    DWORD dwMode=PIPE_READMODE_BYTE | PIPE_WAIT;
    SetNamedPipeHandleState(hp->hPipe, &dwMode, NULL,NULL); //set blocking mode
    DEBUG_OUT << "Connected to named pipe " << hp->pipename << endl;
    return 1;
}


//send data block to pipe
int pipewrite(pipe_t *hp, char *buf, int len)
{
    DWORD cbWritten = 0;
    int res = 0;

    int p=0;
    while(p<len) //is all data sent?
    {
        res = WriteFile( 
            hp->hPipe,  // handle to pipe 
            &buf[p],    // buffer to write from 
            len-p,      // number of bytes to write 
            &cbWritten, // number of bytes written 
            NULL);      // not overlapped I/O 
        if(res)
        {
            p+=cbWritten;
        }
        else
        {
            //disconnected
            DEBUG_OUT << "Pipe " << hp->pipename << " disconnected." << endl;
            return 0;
        }
    }
    return 1;
}

//get data block from pipe
int piperead(pipe_t *hp, char *buf, int len)
{
    DWORD cbRead = 0;
    int res = 0;

    int p=0;
    while(p<len) //is all data recv'd?
    {
        res = ReadFile(
            hp->hPipe,  // handle to pipe
            &buf[p],    // buffer to read to
            len-p,      // number of bytes to read
            &cbRead,    // number of bytes read
            NULL);      // not overlapped I/O
        if(res)
        {
            p+=cbRead;
        }
        else
        {
            //disconnected
            DEBUG_OUT << "Pipe " << hp->pipename << " disconnected." << endl;
            return p;
        }
    }
    return p;
}


//close pipe
void close_pipe(pipe_t *hp)
{
    //close pipe
    DEBUG_OUT << "Close pipe " << hp->pipename << endl;
    CloseHandle(hp->hPipe);
}





//start new process
proc_t* run_proc(char *cmdline, int newcons)
{
    proc_t *pp = new proc_t;
    pp->name = string(cmdline);
    DEBUG_OUT << "Starting process " << pp->name << endl;

    memset(&pp->si, 0, sizeof(pp->si));
    pp->si.cb=sizeof(pp->si);
    //si.wShowWindow=SW_SHOW;
    pp->si.wShowWindow=SW_MINIMIZE;
    //si.wShowWindow=SW_HIDE;
    pp->si.dwFlags=STARTF_USESHOWWINDOW;
    memset(&pp->pi, 0, sizeof(pp->pi));

    if(!CreateProcess(NULL, // no app name
        cmdline,            // cmd line
        NULL,               // proc attr
        NULL,               // thread attr
        FALSE,              // Inherit Handles
        (newcons)?CREATE_NEW_CONSOLE:0, // Creation Flags
        NULL,               // Environment
        NULL,               // Current Directory
        &pp->si,            // Startup Info
        &pp->pi))           // Process Information
    {
        cout << "Error: can not create process." << endl;
        delete pp;
        return NULL;
    }
    DEBUG_OUT << "Process started " << pp->name << endl;
    return pp;
}


//wait for process ends
int wait_proc(proc_t* pp)
{
    DEBUG_OUT << "Waiting for process " << pp->name << endl;
    //wait
    WaitForSingleObject(pp->pi.hProcess, INFINITE);
    int res=0;
    //close handles
    CloseHandle(pp->pi.hProcess);
    CloseHandle(pp->pi.hThread);
    DEBUG_OUT << "Process stopped " << pp->name << endl;
    return res;
}



