#include <cstdio>
#include <iostream>
#include <sstream>
#include <string>
using namespace std;

#include "sys.h"
#include "tgv.h"
#include "debug.h"


int snd_size;
int frames;
FILE *tgv;


//create TGV file, put dummy header
int create_tgv(const char *fn)
{
  DEBUG_OUT << "Creating TGV file " << fn << endl;

  tgv = fopen(fn, "w+b");
  if(!tgv)
  {
    DEBUG_OUT << "Creating TGV file error." << endl;
    return 0;
  }

  //write dummy hdr
  char buf[512]={0};
  fwrite(buf,1,512,tgv);

  snd_size=0;
  frames=0;
  return 1;
}


//write sound data
int write_tgv_snd(char *buf, int len)
{
  int n=fwrite(buf,1,len,tgv);
  
  snd_size+=len;
  return n==len;
}


//finish sound data (sector padding)
int fin_tgv_snd()
{
  if(snd_size%512)
  {
    char buf[512]={0};
    int len=512-(snd_size%512);
    int n=fwrite(buf,1,len,tgv);
    snd_size+=len;
    return n==len;
  }
  DEBUG_OUT << "Sound saved, size=" << snd_size << " bytes, " << snd_size/512 << " sectors." << endl;
  return true;
}


//write video frame data
int write_tgv_frame(char *buf, int len)
{
  int n=fwrite(buf,1,len,tgv);
  
  frames++;
  return n==len;
}


//close file, update TGV header
void fin_tgv()
{
  DEBUG_OUT << "Video saved, " << frames/2 << " frames." << endl;

  //update tgv file header
  struct
  {
    char hdr[16];
    unsigned char ssecl,ssech,res[14];
    unsigned char hl,hh;
  } tgv_hdr;

  memset(&tgv_hdr,0,sizeof(tgv_hdr));
  memcpy(tgv_hdr.hdr,"TGA Video v0.2  ",16);
  tgv_hdr.ssecl=(unsigned char)((snd_size/512)&0xff);
  tgv_hdr.ssech=(unsigned char)((snd_size/512)>>8);
  tgv_hdr.hl=(unsigned char)(TGV_HEIGHT&0xff);
  tgv_hdr.hh=(unsigned char)(TGV_HEIGHT>>8);

  fseek(tgv,0,SEEK_SET);
  fwrite(&tgv_hdr, 1, sizeof(tgv_hdr), tgv);

  fclose(tgv);
  tgv=NULL;

}





FILE *rtgv;
char *rtgv_snd, *rtgv_frame;
int rtgv_snd_size;
int rtgv_frame_h;
int rtgv_frame_size;

//open TGV file for reading
int open_tgv(const char *fn)
{
  DEBUG_OUT << "Open TGV file " << fn << endl;

  rtgv = fopen(fn, "rb");
  if(!rtgv)
  {
    DEBUG_OUT << "Open TGV file error." << endl;
    return 0;
  }

  //check hdr
  unsigned char buf[512];
  fread(buf,1,512,rtgv);
  if(memcmp(buf,"TGA Video v0.2  ",16)!=0)
  {
    DEBUG_OUT << "TGV header error." << endl;
    return 0;
  }
  rtgv_frame_h = buf[32] + 256*buf[33];
  rtgv_frame_size = rtgv_frame_h*TGV_WIDTH*1 +256*2;
  rtgv_snd_size=(buf[16] + 256*buf[17]) * 512;
  rtgv_snd=(char*)malloc(rtgv_snd_size);
  rtgv_frame=(char*)malloc(rtgv_frame_size);

  if(!rtgv_snd || !rtgv_frame)
  {
    DEBUG_OUT << "Malloc error." << endl;
    return 0;
  }

  fread(rtgv_snd,1,rtgv_snd_size,rtgv);
  return 1;
}


//return video frame height
int get_tgv_frame_h()
{
  return rtgv_frame_h;
}


//return ptr to sound data
char* get_tgv_snd(int &size)
{
  size=rtgv_snd_size;
  return rtgv_snd;
}


//return ptr to video frame
char* get_tgv_frame()
{
  int n=fread(rtgv_frame,1,rtgv_frame_size,rtgv);
  if(n==rtgv_frame_size)
    return rtgv_frame;
  else
    return NULL;
}


//close TGV
void close_tgv()
{
  DEBUG_OUT << "Close TGV file." << endl;

  fclose(rtgv);
  rtgv=NULL;
  if(rtgv_snd) free(rtgv_snd);
  if(rtgv_frame) free(rtgv_frame);
  rtgv_snd=NULL;
  rtgv_frame=NULL;
}

